/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * <p>
 * http://www.apache.org/licenses/LICENSE-2.0
 * <p>
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.ambari.view.zeppelin;

import org.apache.ambari.view.ViewContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;


public class ZeppelinServlet extends HttpServlet {
    private ViewContext viewContext;
    private final static Logger LOG = LoggerFactory.getLogger(ZeppelinServlet.class);

    @Override
    public void init(ServletConfig config) throws ServletException {
        super.init(config);

        ServletContext context = config.getServletContext();
        viewContext = (ViewContext) context.getAttribute(ViewContext.CONTEXT_ATTRIBUTE);
    }

    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
        response.setContentType("text/html");
        response.setStatus(HttpServletResponse.SC_OK);
        String publicName = "";
        String port = "";
        try {
            port = viewContext.getProperties().get("zeppelin.server.port");
            if (viewContext.getCluster() != null) {
                List<String> hostsForServiceComponents = viewContext.getCluster().getHostsForServiceComponent("ZEPPELIN", "ZEPPELIN_MASTER");
                publicName = hostsForServiceComponents.get(0);
            } else {
                publicName = viewContext.getProperties().get("zeppelin.host.publicname");
            }
        } catch (Exception e) {
            LOG.error("Zeppelin view servlet failed", e);
        }

        String serviceCheckResponse = ZeppelinServiceCheck.check(publicName, port);
        request.setAttribute("serviceCheckResponse", serviceCheckResponse);

        request.getRequestDispatcher("WEB-INF/index.jsp").forward(request, response);
    }
}
